// Copyright (c) 2003 Logitech, Inc. All Rights Reserved 
/////////////////////////////////////////////////////////////////////////////////
//                                                                             //
// Copyright (c) 2003 Logitech, Inc. All Rights Reserved                       //
//                                                                             //
// This program is a trade secret of LOGITECH, and it is not to be reproduced, //
// published, disclosed to others, copied, adapted, distributed or displayed   //
// without the prior written authorization of LOGITECH.                        //
//                                                                             //
// Licensee agrees to attach or embed this notice on all copies of the program //
// including partial copies or modified versions thereof.                      //
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <cpl.h>
#include <Shellapi.h>
#include <Winreg.h>
#include <TChar.h>
#include "..\\resource\\resource.h"

static const TCHAR IO_REG_PATH[]		= TEXT("Software\\Logitech\\ioSoftware\\2.0\\Paths");
static const TCHAR IO_REG_SETTINGS[]	= TEXT("Software\\Logitech\\ioSoftware\\2.0\\Settings");
static const TCHAR IO_CPL_NAME[]		= TEXT("\\Logitech.Pen.ControlPanel.exe");
static const TCHAR IO_CULTURE[]			= TEXT("Culture");
static const TCHAR IO_PRODUCTDIR[]		= TEXT("ProductDir");
static const TCHAR IO_BRAND[]			= TEXT("Brand");

struct Culture
{
	TCHAR szCode[3];			// Two letter ISO-639-1 code for the culture
    int   iIcon;				// Icon resource identifier
    int   iNameString;			// Name-string resource identifier
    int   iDescString;			// Description-string resource identifier
};

static Culture Branded[] = 
{
	{TEXT("en"), IDI_ICON, IDS_NAME_EN, IDS_DESC_EN },	// English
	{TEXT("fr"), IDI_ICON, IDS_NAME_FR, IDS_DESC_FR },	// French
	{TEXT("de"), IDI_ICON, IDS_NAME_DE, IDS_DESC_DE },	// German
	{TEXT("it"), IDI_ICON, IDS_NAME_IT, IDS_DESC_IT },	// Italian
	{TEXT("es"), IDI_ICON, IDS_NAME_ES, IDS_DESC_ES },	// Spanish
	{TEXT("sv"), IDI_ICON, IDS_NAME_SV, IDS_DESC_SV },	// Swedish
	{0,0,0}
};


static Culture Unbranded[] = 
{
	{TEXT("en"), IDI_ICON, IDS_NAME_UNBRAND_EN, IDS_DESC_UNBRAND_EN },	// English
	{TEXT("fr"), IDI_ICON, IDS_NAME_UNBRAND_FR, IDS_DESC_UNBRAND_FR },	// French
	{TEXT("de"), IDI_ICON, IDS_NAME_UNBRAND_DE, IDS_DESC_UNBRAND_DE },	// German
	{TEXT("it"), IDI_ICON, IDS_NAME_UNBRAND_IT, IDS_DESC_UNBRAND_IT },	// Italian
	{TEXT("es"), IDI_ICON, IDS_NAME_UNBRAND_ES, IDS_DESC_UNBRAND_ES },	// Spanish
	{TEXT("sv"), IDI_ICON, IDS_NAME_UNBRAND_SV, IDS_DESC_UNBRAND_SV },	// Swedish
	{0,0,0}
};

static Culture* FindCulture (Culture cultures[], LPCTSTR szLanguageCode)
{
	for (int i = 0; cultures[i].szCode != 0; ++i) {
		if (0 == ::_tcsicmp(cultures[i].szCode, szLanguageCode))
		 return &cultures[i];
	}
	return &cultures[0];
}

struct ResourceTable
{
	LPCTSTR  szBrandName;
	Culture* cultures;
};

static ResourceTable ResourceTables[] =
{
	{ TEXT(""),			 Branded },
	{ TEXT("Unbranded"), Unbranded },
};

static ResourceTable* FindResourceTable (LPCTSTR szBrandName)
{
	for (int i = 0; sizeof(ResourceTables) / sizeof(ResourceTable); ++i) {
		if (0 == ::_tcsicmp(ResourceTables[i].szBrandName, szBrandName))
		 return &ResourceTables[i];
	}
	return &ResourceTables[0];
}


BOOL APIENTRY DllMain( HANDLE hModule, DWORD ulReason, LPVOID lpReserved ) 
{
	switch ( ulReason ) 
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}

	return TRUE;
}


LONG CALLBACK CPlApplet(HWND hwndCPL, UINT uMessage, LPARAM lParam1, LPARAM lParam2)
{
	HKEY		hKeyRoot, hKeyCulture, hKeyBrand;
	DWORD		dwPathBufferSize		= 256;
	DWORD		dwLanguageBufferSize	= 256;
	DWORD		dwBrandBufferSize		= 256;
	int			iDefaultLanguageIndex	= 0;
	TCHAR		szLanguageCode[256]		= _T("");
	TCHAR		szBrandName[256]		= _T("");
	TCHAR		szProductPath[256]		= _T("");
	Culture*    culture					= 0;

	switch ( uMessage )
	{
		case CPL_INIT:
			return TRUE;

		case CPL_GETCOUNT:
			return 1;

		case CPL_INQUIRE:
			{
				///////////////////////////
				/// Open Language Settings
				///

				if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, IO_REG_SETTINGS, 0, KEY_READ, &hKeyCulture) == ERROR_SUCCESS)
				{
					RegQueryValueEx(hKeyCulture, IO_CULTURE, NULL, NULL, (LPBYTE)szLanguageCode, &dwLanguageBufferSize);
				}
				RegCloseKey(hKeyCulture);

				/////////////////////////
				/// Open Brand Settings
				///

				if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, IO_REG_SETTINGS, 0, KEY_READ, &hKeyBrand) == ERROR_SUCCESS)
				{
					RegQueryValueEx(hKeyBrand, IO_BRAND, NULL, NULL, (LPBYTE)szBrandName, &dwBrandBufferSize);
				}
				RegCloseKey(hKeyBrand);

				/////////////////////////////////
				// Find the Equivalent resource.
				//
				ResourceTable* resourceTable = FindResourceTable(szBrandName);
				Culture* culture = FindCulture(resourceTable->cultures, szLanguageCode);

				LPCPLINFO lpCPlInfo = (LPCPLINFO) lParam2;
				lpCPlInfo->lData  = 0;
				lpCPlInfo->idIcon = culture->iIcon;
				lpCPlInfo->idName = culture->iNameString;
				lpCPlInfo->idInfo = culture->iDescString;

				break;
			}

		case CPL_DBLCLK:
			{
				if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, IO_REG_PATH, 0, KEY_QUERY_VALUE, &hKeyRoot) == ERROR_SUCCESS)
				{
					if (RegQueryValueEx(hKeyRoot, IO_PRODUCTDIR, NULL, NULL, (LPBYTE) szProductPath, &dwPathBufferSize) == ERROR_SUCCESS)
					{
						::_tcscat(szProductPath, IO_CPL_NAME);
						ShellExecute(hwndCPL, "open", szProductPath,"", "", SW_SHOW);
					}
					else
					{
						MessageBox(hwndCPL, "Unable to locate \"Software\\Logitech\\ioSoftware\\2.0\\Paths\" registry key. Please contact administrator.", "", MB_OK | MB_ICONERROR);
					}
					
					RegCloseKey(hKeyRoot);
				}
				else
				{
					MessageBox(hwndCPL, "Unable to locate \"Software\\Logitech\\ioSoftware\\2.0\\Paths\" registry key. Please contact administrator.", "", MB_OK | MB_ICONERROR);
				}

				break;
			}

		case CPL_STOP:
			break;

		case CPL_EXIT:
			break;

		default:
			break;
	}

	return 0;
}